# Copyright 2018 ACSONE SA/NV (<http://acsone.eu>)
# License LGPL-3.0 or later (http://www.gnu.org/licenses/lgpl.html).

from contextlib import contextmanager
import logging
import sys

import click

try:
    import odoo
    from odoo.api import Environment
except ImportError:
    # Odoo < 10
    try:
        import openerp as odoo
        from openerp.api import Environment
    except ImportError:
        raise ImportError("No module named odoo nor openerp")

_logger = logging.getLogger(__name__)


def _fix_logging(series):
    if series < 9:
        handlers = logging.getLogger().handlers
        if handlers and len(handlers) == 1:
            handler = handlers[0]
            if isinstance(handler, logging.StreamHandler):
                if handler.stream is sys.stdout:
                    handler.stream = sys.stderr


@contextmanager
def OdooEnvironment(config=None, database=None, log_level=None, logfile=None,
                    rollback=False):
    series = odoo.release.version_info[0]

    odoo_args = []
    if config:
        odoo_args.extend(['-c', config])
    if database:
        odoo_args.extend(['-d', database])
    if log_level:
        odoo_args.extend(['--log-level', log_level])
    if logfile:
        odoo_args.extend(['--logfile', logfile])
    # see https://github.com/odoo/odoo/commit/b122217f74
    odoo.tools.config['load_language'] = None
    odoo.tools.config.parse_config(odoo_args)

    _fix_logging(series)

    db_name = odoo.tools.config['db_name']
    if not db_name:
        raise click.ClickException("No database name found. Please provide "
                                   "one with the -d option or the odoo "
                                   "configuration file.")

    with Environment.manage():
        registry = odoo.registry(db_name)
        registry.clear_caches()
        with registry.cursor() as cr:
            uid = odoo.SUPERUSER_ID
            ctx = Environment(cr, uid, {})['res.users'].context_get()
            env = Environment(cr, uid, ctx)
            cr.rollback()
            try:
                yield env
                if rollback:
                    cr.rollback()
                else:
                    cr.commit()
            except:  # noqa
                _logger.exception("Exception in OdooEnvironment")
                raise



#!/usr/bin/env python
import os
import subprocess
import sys

import psycopg2


odoo_series = sys.argv[1]
odoo_dir = os.path.join(os.environ['VIRTUAL_ENV'], 'src', 'odoo')
# see also dbname in ../test_click_odoo.py
odoo_db = 'click-odoo-test-' + odoo_series.replace('.', '-')
if odoo_series in ('8.0', '9.0'):
    odoo_bin = 'openerp-server'
else:
    odoo_bin = 'odoo'


def odoo_installed():
    try:
        import odoo  # noqa
        return True
    except ImportError:
        # odoo < 10
        try:
            import openerp  # noqa
            return True
        except ImportError:
            # odoo not installed
            return False


def odoo_cloned():
    return os.path.isdir(os.path.join(odoo_dir, '.git'))


def clone_odoo():
    subprocess.check_call([
        'git', 'clone',
        '--depth=1',
        '-b', odoo_series,
        'https://github.com/odoo/odoo',
        odoo_dir,
    ])


def install_odoo():
    subprocess.check_call([
        'pip', 'install',
        '-e', odoo_dir,
    ])


def create_db():
    # createdb if it does not exist
    try:
        with psycopg2.connect(dbname=odoo_db) as conn:
            pass
    except psycopg2.OperationalError:
        subprocess.check_call(['createdb', odoo_db])
    # check if db initialized
    with psycopg2.connect(dbname=odoo_db) as conn:
        with conn.cursor() as cr:
            cr.execute("""
                select * from information_schema.tables
                where table_name='ir_module_module'
            """)
            if cr.rowcount:
                return
    # initialize database
    subprocess.check_call([
        odoo_bin,
        '-d', odoo_db,
        '--stop-after-init',
    ])


def main():
    if not odoo_installed():
        if not odoo_cloned():
            clone_odoo()
        install_odoo()
    create_db()


if __name__ == '__main__':
    main()
